import datetime
from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.agent_version_variance_response import AgentVersionVarianceResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    suite_id: int | None | Unset = UNSET,
    variance_threshold: float | Unset = 0.1,
    limit: int | Unset = 20,
    since: datetime.datetime | None | Unset = UNSET,
    until: datetime.datetime | None | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_suite_id: int | None | Unset
    if isinstance(suite_id, Unset):
        json_suite_id = UNSET
    else:
        json_suite_id = suite_id
    params["suite_id"] = json_suite_id

    params["variance_threshold"] = variance_threshold

    params["limit"] = limit

    json_since: None | str | Unset
    if isinstance(since, Unset):
        json_since = UNSET
    elif isinstance(since, datetime.datetime):
        json_since = since.isoformat()
    else:
        json_since = since
    params["since"] = json_since

    json_until: None | str | Unset
    if isinstance(until, Unset):
        json_until = UNSET
    elif isinstance(until, datetime.datetime):
        json_until = until.isoformat()
    else:
        json_until = until
    params["until"] = json_until

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/admin/analytics/agent-version-variance",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> AgentVersionVarianceResponse | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = AgentVersionVarianceResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[AgentVersionVarianceResponse | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    suite_id: int | None | Unset = UNSET,
    variance_threshold: float | Unset = 0.1,
    limit: int | Unset = 20,
    since: datetime.datetime | None | Unset = UNSET,
    until: datetime.datetime | None | Unset = UNSET,
) -> Response[AgentVersionVarianceResponse | HTTPValidationError]:
    """Score variance across validators for recent agent versions

     Find agent versions with high score variance across validators.

    Args:
        suite_id (int | None | Unset): Suite ID (defaults to active suite)
        variance_threshold (float | Unset): Spread threshold for flagging Default: 0.1.
        limit (int | Unset): Number of agent versions Default: 20.
        since (datetime.datetime | None | Unset): Only versions after this time
        until (datetime.datetime | None | Unset): Only versions before this time

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AgentVersionVarianceResponse | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        suite_id=suite_id,
        variance_threshold=variance_threshold,
        limit=limit,
        since=since,
        until=until,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    suite_id: int | None | Unset = UNSET,
    variance_threshold: float | Unset = 0.1,
    limit: int | Unset = 20,
    since: datetime.datetime | None | Unset = UNSET,
    until: datetime.datetime | None | Unset = UNSET,
) -> AgentVersionVarianceResponse | HTTPValidationError | None:
    """Score variance across validators for recent agent versions

     Find agent versions with high score variance across validators.

    Args:
        suite_id (int | None | Unset): Suite ID (defaults to active suite)
        variance_threshold (float | Unset): Spread threshold for flagging Default: 0.1.
        limit (int | Unset): Number of agent versions Default: 20.
        since (datetime.datetime | None | Unset): Only versions after this time
        until (datetime.datetime | None | Unset): Only versions before this time

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AgentVersionVarianceResponse | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        suite_id=suite_id,
        variance_threshold=variance_threshold,
        limit=limit,
        since=since,
        until=until,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    suite_id: int | None | Unset = UNSET,
    variance_threshold: float | Unset = 0.1,
    limit: int | Unset = 20,
    since: datetime.datetime | None | Unset = UNSET,
    until: datetime.datetime | None | Unset = UNSET,
) -> Response[AgentVersionVarianceResponse | HTTPValidationError]:
    """Score variance across validators for recent agent versions

     Find agent versions with high score variance across validators.

    Args:
        suite_id (int | None | Unset): Suite ID (defaults to active suite)
        variance_threshold (float | Unset): Spread threshold for flagging Default: 0.1.
        limit (int | Unset): Number of agent versions Default: 20.
        since (datetime.datetime | None | Unset): Only versions after this time
        until (datetime.datetime | None | Unset): Only versions before this time

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AgentVersionVarianceResponse | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        suite_id=suite_id,
        variance_threshold=variance_threshold,
        limit=limit,
        since=since,
        until=until,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    suite_id: int | None | Unset = UNSET,
    variance_threshold: float | Unset = 0.1,
    limit: int | Unset = 20,
    since: datetime.datetime | None | Unset = UNSET,
    until: datetime.datetime | None | Unset = UNSET,
) -> AgentVersionVarianceResponse | HTTPValidationError | None:
    """Score variance across validators for recent agent versions

     Find agent versions with high score variance across validators.

    Args:
        suite_id (int | None | Unset): Suite ID (defaults to active suite)
        variance_threshold (float | Unset): Spread threshold for flagging Default: 0.1.
        limit (int | Unset): Number of agent versions Default: 20.
        since (datetime.datetime | None | Unset): Only versions after this time
        until (datetime.datetime | None | Unset): Only versions before this time

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AgentVersionVarianceResponse | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            suite_id=suite_id,
            variance_threshold=variance_threshold,
            limit=limit,
            since=since,
            until=until,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="ValidatorScoreSummary")


@_attrs_define
class ValidatorScoreSummary:
    """Aggregated scoring stats for a single validator.

    Attributes:
        validator_hotkey (str): Validator hotkey
        total_runs (int): Total completed runs
        avg_score (float): Mean score across runs
        median_score (float): Median score
        stddev_score (float): Standard deviation of scores
        min_score (float): Minimum score
        max_score (float): Maximum score
        deviation_from_global (float): Percentage deviation from the global average (negative = below)
        is_outlier (bool): True if deviation exceeds 1.5 standard deviations from global mean
    """

    validator_hotkey: str
    total_runs: int
    avg_score: float
    median_score: float
    stddev_score: float
    min_score: float
    max_score: float
    deviation_from_global: float
    is_outlier: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        validator_hotkey = self.validator_hotkey

        total_runs = self.total_runs

        avg_score = self.avg_score

        median_score = self.median_score

        stddev_score = self.stddev_score

        min_score = self.min_score

        max_score = self.max_score

        deviation_from_global = self.deviation_from_global

        is_outlier = self.is_outlier

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "validator_hotkey": validator_hotkey,
                "total_runs": total_runs,
                "avg_score": avg_score,
                "median_score": median_score,
                "stddev_score": stddev_score,
                "min_score": min_score,
                "max_score": max_score,
                "deviation_from_global": deviation_from_global,
                "is_outlier": is_outlier,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        validator_hotkey = d.pop("validator_hotkey")

        total_runs = d.pop("total_runs")

        avg_score = d.pop("avg_score")

        median_score = d.pop("median_score")

        stddev_score = d.pop("stddev_score")

        min_score = d.pop("min_score")

        max_score = d.pop("max_score")

        deviation_from_global = d.pop("deviation_from_global")

        is_outlier = d.pop("is_outlier")

        validator_score_summary = cls(
            validator_hotkey=validator_hotkey,
            total_runs=total_runs,
            avg_score=avg_score,
            median_score=median_score,
            stddev_score=stddev_score,
            min_score=min_score,
            max_score=max_score,
            deviation_from_global=deviation_from_global,
            is_outlier=is_outlier,
        )

        validator_score_summary.additional_properties = d
        return validator_score_summary

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AdminAgentVersionEntry")


@_attrs_define
class AdminAgentVersionEntry:
    """An agent version entry for admin listing.

    Attributes:
        agent_version_id (UUID): Agent version ID
        agent_id (UUID): Parent agent ID
        agent_name (str): Agent name
        miner_hotkey (str): Miner hotkey that owns this agent
        version_number (int): Version number
        created_at (datetime.datetime): When this version was created
        is_eligible (bool | None | Unset): Whether version is eligible
        is_discarded (bool | None | Unset): Whether version is discarded
        final_score (float | None | Unset): Final aggregate score
        work_item_is_closed (bool | None | Unset): Whether the work item is closed
        work_item_is_cancelled (bool | None | Unset): Whether the work item is cancelled
        active_run_count (int | None | Unset): Number of active evaluation runs
        is_current_top (bool | Unset): Whether this is the current top agent Default: False.
        work_item_included_success_count (int | None | Unset): Number of included successful runs
    """

    agent_version_id: UUID
    agent_id: UUID
    agent_name: str
    miner_hotkey: str
    version_number: int
    created_at: datetime.datetime
    is_eligible: bool | None | Unset = UNSET
    is_discarded: bool | None | Unset = UNSET
    final_score: float | None | Unset = UNSET
    work_item_is_closed: bool | None | Unset = UNSET
    work_item_is_cancelled: bool | None | Unset = UNSET
    active_run_count: int | None | Unset = UNSET
    is_current_top: bool | Unset = False
    work_item_included_success_count: int | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        agent_version_id = str(self.agent_version_id)

        agent_id = str(self.agent_id)

        agent_name = self.agent_name

        miner_hotkey = self.miner_hotkey

        version_number = self.version_number

        created_at = self.created_at.isoformat()

        is_eligible: bool | None | Unset
        if isinstance(self.is_eligible, Unset):
            is_eligible = UNSET
        else:
            is_eligible = self.is_eligible

        is_discarded: bool | None | Unset
        if isinstance(self.is_discarded, Unset):
            is_discarded = UNSET
        else:
            is_discarded = self.is_discarded

        final_score: float | None | Unset
        if isinstance(self.final_score, Unset):
            final_score = UNSET
        else:
            final_score = self.final_score

        work_item_is_closed: bool | None | Unset
        if isinstance(self.work_item_is_closed, Unset):
            work_item_is_closed = UNSET
        else:
            work_item_is_closed = self.work_item_is_closed

        work_item_is_cancelled: bool | None | Unset
        if isinstance(self.work_item_is_cancelled, Unset):
            work_item_is_cancelled = UNSET
        else:
            work_item_is_cancelled = self.work_item_is_cancelled

        active_run_count: int | None | Unset
        if isinstance(self.active_run_count, Unset):
            active_run_count = UNSET
        else:
            active_run_count = self.active_run_count

        is_current_top = self.is_current_top

        work_item_included_success_count: int | None | Unset
        if isinstance(self.work_item_included_success_count, Unset):
            work_item_included_success_count = UNSET
        else:
            work_item_included_success_count = self.work_item_included_success_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "agent_version_id": agent_version_id,
                "agent_id": agent_id,
                "agent_name": agent_name,
                "miner_hotkey": miner_hotkey,
                "version_number": version_number,
                "created_at": created_at,
            }
        )
        if is_eligible is not UNSET:
            field_dict["is_eligible"] = is_eligible
        if is_discarded is not UNSET:
            field_dict["is_discarded"] = is_discarded
        if final_score is not UNSET:
            field_dict["final_score"] = final_score
        if work_item_is_closed is not UNSET:
            field_dict["work_item_is_closed"] = work_item_is_closed
        if work_item_is_cancelled is not UNSET:
            field_dict["work_item_is_cancelled"] = work_item_is_cancelled
        if active_run_count is not UNSET:
            field_dict["active_run_count"] = active_run_count
        if is_current_top is not UNSET:
            field_dict["is_current_top"] = is_current_top
        if work_item_included_success_count is not UNSET:
            field_dict["work_item_included_success_count"] = work_item_included_success_count

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        agent_version_id = UUID(d.pop("agent_version_id"))

        agent_id = UUID(d.pop("agent_id"))

        agent_name = d.pop("agent_name")

        miner_hotkey = d.pop("miner_hotkey")

        version_number = d.pop("version_number")

        created_at = isoparse(d.pop("created_at"))

        def _parse_is_eligible(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        is_eligible = _parse_is_eligible(d.pop("is_eligible", UNSET))

        def _parse_is_discarded(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        is_discarded = _parse_is_discarded(d.pop("is_discarded", UNSET))

        def _parse_final_score(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        final_score = _parse_final_score(d.pop("final_score", UNSET))

        def _parse_work_item_is_closed(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        work_item_is_closed = _parse_work_item_is_closed(d.pop("work_item_is_closed", UNSET))

        def _parse_work_item_is_cancelled(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        work_item_is_cancelled = _parse_work_item_is_cancelled(d.pop("work_item_is_cancelled", UNSET))

        def _parse_active_run_count(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        active_run_count = _parse_active_run_count(d.pop("active_run_count", UNSET))

        is_current_top = d.pop("is_current_top", UNSET)

        def _parse_work_item_included_success_count(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        work_item_included_success_count = _parse_work_item_included_success_count(
            d.pop("work_item_included_success_count", UNSET)
        )

        admin_agent_version_entry = cls(
            agent_version_id=agent_version_id,
            agent_id=agent_id,
            agent_name=agent_name,
            miner_hotkey=miner_hotkey,
            version_number=version_number,
            created_at=created_at,
            is_eligible=is_eligible,
            is_discarded=is_discarded,
            final_score=final_score,
            work_item_is_closed=work_item_is_closed,
            work_item_is_cancelled=work_item_is_cancelled,
            active_run_count=active_run_count,
            is_current_top=is_current_top,
            work_item_included_success_count=work_item_included_success_count,
        )

        admin_agent_version_entry.additional_properties = d
        return admin_agent_version_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="AgentVersionScoreEntry")


@_attrs_define
class AgentVersionScoreEntry:
    """Per-validator score for an agent version.

    Attributes:
        validator_hotkey (str): Validator hotkey
        score (float): Score from this validator
        run_id (UUID): Evaluation run ID
    """

    validator_hotkey: str
    score: float
    run_id: UUID
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        validator_hotkey = self.validator_hotkey

        score = self.score

        run_id = str(self.run_id)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "validator_hotkey": validator_hotkey,
                "score": score,
                "run_id": run_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        validator_hotkey = d.pop("validator_hotkey")

        score = d.pop("score")

        run_id = UUID(d.pop("run_id"))

        agent_version_score_entry = cls(
            validator_hotkey=validator_hotkey,
            score=score,
            run_id=run_id,
        )

        agent_version_score_entry.additional_properties = d
        return agent_version_score_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="AdminEvaluationRunEntry")


@_attrs_define
class AdminEvaluationRunEntry:
    """An evaluation run entry for admin listing.

    Attributes:
        run_id (UUID): Evaluation run ID
        agent_version_id (UUID): Agent version being evaluated
        validator_hotkey (str): Validator performing evaluation
        suite_id (int): Problem suite version
        status (str): Run status
        is_included (bool): Whether run is included in aggregate scoring
        created_at (datetime.datetime): When the run was created
        claimed_at (datetime.datetime | None | Unset): When the run was claimed
        completed_at (datetime.datetime | None | Unset): When the run completed
        last_heartbeat_at (datetime.datetime | None | Unset): Last heartbeat timestamp
        lease_expires_at (datetime.datetime | None | Unset): Lease expiration
        score (float | None | Unset): Run score
        invalidated_at (datetime.datetime | None | Unset): When the run was invalidated
        invalidation_reason (None | str | Unset): Reason for invalidation
    """

    run_id: UUID
    agent_version_id: UUID
    validator_hotkey: str
    suite_id: int
    status: str
    is_included: bool
    created_at: datetime.datetime
    claimed_at: datetime.datetime | None | Unset = UNSET
    completed_at: datetime.datetime | None | Unset = UNSET
    last_heartbeat_at: datetime.datetime | None | Unset = UNSET
    lease_expires_at: datetime.datetime | None | Unset = UNSET
    score: float | None | Unset = UNSET
    invalidated_at: datetime.datetime | None | Unset = UNSET
    invalidation_reason: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        run_id = str(self.run_id)

        agent_version_id = str(self.agent_version_id)

        validator_hotkey = self.validator_hotkey

        suite_id = self.suite_id

        status = self.status

        is_included = self.is_included

        created_at = self.created_at.isoformat()

        claimed_at: None | str | Unset
        if isinstance(self.claimed_at, Unset):
            claimed_at = UNSET
        elif isinstance(self.claimed_at, datetime.datetime):
            claimed_at = self.claimed_at.isoformat()
        else:
            claimed_at = self.claimed_at

        completed_at: None | str | Unset
        if isinstance(self.completed_at, Unset):
            completed_at = UNSET
        elif isinstance(self.completed_at, datetime.datetime):
            completed_at = self.completed_at.isoformat()
        else:
            completed_at = self.completed_at

        last_heartbeat_at: None | str | Unset
        if isinstance(self.last_heartbeat_at, Unset):
            last_heartbeat_at = UNSET
        elif isinstance(self.last_heartbeat_at, datetime.datetime):
            last_heartbeat_at = self.last_heartbeat_at.isoformat()
        else:
            last_heartbeat_at = self.last_heartbeat_at

        lease_expires_at: None | str | Unset
        if isinstance(self.lease_expires_at, Unset):
            lease_expires_at = UNSET
        elif isinstance(self.lease_expires_at, datetime.datetime):
            lease_expires_at = self.lease_expires_at.isoformat()
        else:
            lease_expires_at = self.lease_expires_at

        score: float | None | Unset
        if isinstance(self.score, Unset):
            score = UNSET
        else:
            score = self.score

        invalidated_at: None | str | Unset
        if isinstance(self.invalidated_at, Unset):
            invalidated_at = UNSET
        elif isinstance(self.invalidated_at, datetime.datetime):
            invalidated_at = self.invalidated_at.isoformat()
        else:
            invalidated_at = self.invalidated_at

        invalidation_reason: None | str | Unset
        if isinstance(self.invalidation_reason, Unset):
            invalidation_reason = UNSET
        else:
            invalidation_reason = self.invalidation_reason

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "run_id": run_id,
                "agent_version_id": agent_version_id,
                "validator_hotkey": validator_hotkey,
                "suite_id": suite_id,
                "status": status,
                "is_included": is_included,
                "created_at": created_at,
            }
        )
        if claimed_at is not UNSET:
            field_dict["claimed_at"] = claimed_at
        if completed_at is not UNSET:
            field_dict["completed_at"] = completed_at
        if last_heartbeat_at is not UNSET:
            field_dict["last_heartbeat_at"] = last_heartbeat_at
        if lease_expires_at is not UNSET:
            field_dict["lease_expires_at"] = lease_expires_at
        if score is not UNSET:
            field_dict["score"] = score
        if invalidated_at is not UNSET:
            field_dict["invalidated_at"] = invalidated_at
        if invalidation_reason is not UNSET:
            field_dict["invalidation_reason"] = invalidation_reason

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        run_id = UUID(d.pop("run_id"))

        agent_version_id = UUID(d.pop("agent_version_id"))

        validator_hotkey = d.pop("validator_hotkey")

        suite_id = d.pop("suite_id")

        status = d.pop("status")

        is_included = d.pop("is_included")

        created_at = isoparse(d.pop("created_at"))

        def _parse_claimed_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                claimed_at_type_0 = isoparse(data)

                return claimed_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        claimed_at = _parse_claimed_at(d.pop("claimed_at", UNSET))

        def _parse_completed_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                completed_at_type_0 = isoparse(data)

                return completed_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        completed_at = _parse_completed_at(d.pop("completed_at", UNSET))

        def _parse_last_heartbeat_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                last_heartbeat_at_type_0 = isoparse(data)

                return last_heartbeat_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        last_heartbeat_at = _parse_last_heartbeat_at(d.pop("last_heartbeat_at", UNSET))

        def _parse_lease_expires_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                lease_expires_at_type_0 = isoparse(data)

                return lease_expires_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        lease_expires_at = _parse_lease_expires_at(d.pop("lease_expires_at", UNSET))

        def _parse_score(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        score = _parse_score(d.pop("score", UNSET))

        def _parse_invalidated_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                invalidated_at_type_0 = isoparse(data)

                return invalidated_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        invalidated_at = _parse_invalidated_at(d.pop("invalidated_at", UNSET))

        def _parse_invalidation_reason(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        invalidation_reason = _parse_invalidation_reason(d.pop("invalidation_reason", UNSET))

        admin_evaluation_run_entry = cls(
            run_id=run_id,
            agent_version_id=agent_version_id,
            validator_hotkey=validator_hotkey,
            suite_id=suite_id,
            status=status,
            is_included=is_included,
            created_at=created_at,
            claimed_at=claimed_at,
            completed_at=completed_at,
            last_heartbeat_at=last_heartbeat_at,
            lease_expires_at=lease_expires_at,
            score=score,
            invalidated_at=invalidated_at,
            invalidation_reason=invalidation_reason,
        )

        admin_evaluation_run_entry.additional_properties = d
        return admin_evaluation_run_entry

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
